@@ -39,6 +39,7 @@ __all__ = [
     'DateTimeField', 'BooleanField', 'DecimalField', 'FloatField', 'IntegerField',
     'PrimaryKeyField', 'ForeignKeyField', 'DoubleField', 'BigIntegerField', 'Model',
     'filter_query', 'annotate_query', 'F', 'R', 'DateField', 'TimeField',
+    'transaction',
 ]
 
 class ImproperlyConfigured(Exception):
@@ -317,10 +318,10 @@ class Database(object):
     def get_cursor(self):
         return self.get_conn().cursor()
 
-    def execute(self, sql, params=None):
+    def execute(self, sql, params=None, require_commit=True):
         cursor = self.get_cursor()
         res = cursor.execute(sql, params or ())
-        if self.get_autocommit():
+        if require_commit and self.get_autocommit():
             self.commit()
         logger.debug((sql, params))
         return cursor
@@ -471,6 +472,9 @@ class Database(object):
     def sequence_exists(self, sequence):
         raise NotImplementedError
 
+    def transaction(self):
+        return transaction(self)
+
 
 class SqliteDatabase(Database):
     def __init__(self, database, **connect_kwargs):
@@ -524,6 +528,10 @@ class PostgresqlDatabase(Database):
                 AND relname=%s""", (sequence,))
         return bool(res.fetchone()[0])
 
+    def set_search_path(self, *search_path):
+        path_params = ','.join(['%s'] * len(search_path))
+        self.execute('SET search_path TO %s' % path_params, search_path)
+
 
 class MySQLDatabase(Database):
     def __init__(self, database, **connect_kwargs):
@@ -572,6 +580,22 @@ class MySQLDatabase(Database):
         return [r[0] for r in res.fetchall()]
 
 
+class transaction(object):
+    def __init__(self, db):
+        self.db = db
+
+    def __enter__(self):
+        self._orig = self.db.get_autocommit()
+        self.db.set_autocommit(False)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type:
+            self.db.rollback()
+        else:
+            self.db.commit()
+        self.db.set_autocommit(self._orig)
+
+
 class QueryResultWrapper(object):
     """
     Provides an iterator over the results of a raw Query, additionally doing
@@ -586,6 +610,10 @@ class QueryResultWrapper(object):
         self.column_meta = self.query_meta.get('columns')
         self.join_meta = self.query_meta.get('graph')
 
+        # a query will be considered "simple" if it pulls columns straight
+        # from the primary model being queried
+        self.simple = self.query_meta.get('simple') or not self.column_meta
+
         self.__ct = 0
         self.__idx = 0
 
@@ -607,7 +635,7 @@ class QueryResultWrapper(object):
             for i, value in enumerate(row))
 
     def construct_instance(self, row):
-        if not self.column_meta:
+        if self.simple:
             # use attribute names pulled from the result cursor description,
             # and do not attempt to follow joined models
             row_dict = self._row_to_dict(row)
@@ -707,6 +735,7 @@ class QueryResultWrapper(object):
             return inst
 
         instance = self.iterate()
+        instance.prepared() # <-- model prepared hook
         self._result_cache.append(instance)
         self.__ct += 1
         self.__idx += 1
@@ -929,6 +958,7 @@ class EmptyResultException(Exception):
 class BaseQuery(object):
     query_separator = '__'
     force_alias = False
+    require_commit = True
 
     def __init__(self, model):
         self.model = model
@@ -1086,7 +1116,7 @@ class BaseQuery(object):
         seen = seen or set()
 
         if current not in self._joins:
-            return computed
+            return computed, alias_count
 
         for i, (model, join_type, on) in enumerate(self._joins[current]):
             seen.add(model)
@@ -1126,9 +1156,10 @@ class BaseQuery(object):
                 )
             )
 
-            computed.extend(self.follow_joins(model, alias_map, alias_required, alias_count, seen))
+            joins, alias_count = self.follow_joins(model, alias_map, alias_required, alias_count, seen)
+            computed.extend(joins)
 
-        return computed
+        return computed, alias_count
 
     def compile_where(self):
         alias_count = 0
@@ -1144,7 +1175,7 @@ class BaseQuery(object):
         else:
             alias_map[self.model] = ''
 
-        computed_joins = self.follow_joins(self.model, alias_map, alias_required, alias_count)
+        computed_joins, _ = self.follow_joins(self.model, alias_map, alias_required, alias_count)
 
         clauses = [self.parse_node(node, alias_map) for node in self._where]
 
@@ -1253,7 +1284,7 @@ class BaseQuery(object):
         raise NotImplementedError
 
     def raw_execute(self, query, params):
-        return self.database.execute(query, params)
+        return self.database.execute(query, params, self.require_commit)
 
 
 class RawQuery(BaseQuery):
@@ -1285,6 +1316,8 @@ class RawQuery(BaseQuery):
 
 
 class SelectQuery(BaseQuery):
+    require_commit = False
+
     def __init__(self, model, query=None):
         self.query = query or '*'
         self._group_by = []
@@ -1451,6 +1484,8 @@ class SelectQuery(BaseQuery):
 
     def parse_select_query(self, alias_map):
         q = self.query
+        models_queried = 0
+        local_columns = True
 
         if isinstance(q, (list, tuple)):
             q = {self.model: self.query}
@@ -1461,7 +1496,7 @@ class SelectQuery(BaseQuery):
             elif q in (self.model._meta.pk_col, self.model._meta.pk_name):
                 q = {self.model: [self.model._meta.pk_name]}
             else:
-                return q, []
+                return q, [], False
 
         # by now we should have a dictionary if a valid type was passed in
         if not isinstance(q, dict):
@@ -1478,6 +1513,8 @@ class SelectQuery(BaseQuery):
             if model not in q:
                 continue
 
+            models_queried += 1
+
             if '*' in q[model]:
                 idx = q[model].index('*')
                 q[model] =  q[model][:idx] + model._meta.get_field_names() + q[model][idx+1:]
@@ -1487,6 +1524,7 @@ class SelectQuery(BaseQuery):
                     clause = clause.sql_select()
 
                 if isinstance(clause, tuple):
+                    local_columns = False
                     if len(clause) == 3:
                         func, col_name, col_alias = clause
                         column = model._meta.get_column(col_name)
@@ -1508,8 +1546,7 @@ class SelectQuery(BaseQuery):
                     columns.append(self.safe_combine(model, alias, column))
                     model_cols.append((model, column))
 
-        return ', '.join(columns), model_cols
-
+        return ', '.join(columns), model_cols, (models_queried == 1 and local_columns)
 
     def sql_meta(self):
         joins, clauses, alias_map = self.compile_where()
@@ -1533,10 +1570,11 @@ class SelectQuery(BaseQuery):
             for field in clause:
                 group_by.append(self.safe_combine(model, alias, field))
 
-        parsed_query, model_cols = self.parse_select_query(alias_map)
+        parsed_query, model_cols, simple = self.parse_select_query(alias_map)
         query_meta = {
             'columns': model_cols,
             'graph': self._joins,
+            'simple': simple,
         }
 
         if self._distinct:
@@ -1652,7 +1690,7 @@ class UpdateQuery(BaseQuery):
 
         alias = alias_map.get(self.model)
 
-        for k, v in set_statement.iteritems():
+        for k, v in sorted(set_statement.items(), key=lambda (k, v): k):
             if isinstance(v, F):
                 value = self.parse_f(v, v.model or self.model, alias_map)
             else:
@@ -1724,7 +1762,7 @@ class InsertQuery(BaseQuery):
     def parse_insert(self):
         cols = []
         vals = []
-        for k, v in self.insert_query.iteritems():
+        for k, v in sorted(self.insert_query.items(), key=lambda (k, v): k):
             if k in self.model._meta.columns:
                 k = self.model._meta.columns[k].name
 
@@ -1958,32 +1996,50 @@ class TextColumn(Column):
     def db_value(self, value):
         return value or ''
 
+def format_date_time(value, formats, post_process=None):
+    post_process = post_process or (lambda x: x)
+    for fmt in formats:
+        try:
+            return post_process(datetime.datetime.strptime(value, fmt))
+        except ValueError:
+            pass
+    return value
+
 
 class DateTimeColumn(Column):
     db_field = 'datetime'
-    dt_re = re.compile('^(\d{4}-\d{1,2}-\d{1,2} \d{2}:\d{2}:\d{2})(?:\.(\d+))?')
-    dt_bare_re = re.compile('^\d{4}-\d{1,2}-\d{1,2}$')
+
+    def get_attributes(self):
+        return {
+            'formats': [
+                '%Y-%m-%d %H:%M:%S.%f',
+                '%Y-%m-%d %H:%M:%S',
+                '%Y-%m-%d',
+            ]
+        }
 
     def python_value(self, value):
         if isinstance(value, basestring):
-            match = self.dt_re.match(value)
-            if match:
-                value, usec = match.groups()
-                kwargs = dict(microsecond=usec and int(usec) or 0)
-                return datetime.datetime(*time.strptime(value, '%Y-%m-%d %H:%M:%S')[:6], **kwargs)
-            elif self.dt_bare_re.match(value):
-                return datetime.datetime(*time.strptime(value, '%Y-%m-%d')[:3])
+            return format_date_time(value, self.attributes['formats'])
         return value
 
 
 class DateColumn(Column):
     db_field = 'date'
-    dt_bare_re = re.compile('^\d{4}-\d{1,2}-\d{1,2}$')
+
+    def get_attributes(self):
+        return {
+            'formats': [
+                '%Y-%m-%d',
+                '%Y-%m-%d %H:%M:%S',
+                '%Y-%m-%d %H:%M:%S.%f',
+            ]
+        }
 
     def python_value(self, value):
         if isinstance(value, basestring):
-            if self.dt_bare_re.match(value):
-                return datetime.date(*time.strptime(value, '%Y-%m-%d')[:3])
+            pp = lambda x: x.date()
+            return format_date_time(value, self.attributes['formats'], pp)
         elif isinstance(value, datetime.datetime):
             return value.date()
         return value
@@ -1991,15 +2047,22 @@ class DateColumn(Column):
 
 class TimeColumn(Column):
     db_field = 'time'
-    time_re = re.compile('^(\d{2}:\d{2}:\d{2})(?:\.(\d+))?$')
+
+    def get_attributes(self):
+        return {
+            'formats': [
+                '%H:%M:%S.%f',
+                '%H:%M:%S',
+                '%H:%M',
+                '%Y-%m-%d %H:%M:%S.%f',
+                '%Y-%m-%d %H:%M:%S',
+            ]
+        }
 
     def python_value(self, value):
         if isinstance(value, basestring):
-            match = self.time_re.match(value)
-            if match:
-                value, usec = match.groups()
-                kwargs = dict(microsecond=usec and int(usec) or 0)
-                return datetime.time(*time.strptime(value, '%H:%M:%S')[3:6], **kwargs)
+            pp = lambda x: x.time()
+            return format_date_time(value, self.attributes['formats'], pp)
         elif isinstance(value, datetime.datetime):
             return value.time()
         return value
@@ -2027,7 +2090,8 @@ class BooleanColumn(Column):
         return bool(value)
 
     def python_value(self, value):
-        return bool(value)
+        if value is not None:
+            return bool(value)
 
 
 class FloatColumn(Column):
@@ -2501,11 +2565,19 @@ class Model(object):
                     field_value = field.default
                 setattr(self, field.name, field_value)
 
+    def prepared(self):
+        # this hook is called when the model has been populated from a db cursor
+        pass
+
     def __eq__(self, other):
         return other.__class__ == self.__class__ and \
                self.get_pk() and \
                other.get_pk() == self.get_pk()
 
+    def __ne__(self, other):
+        return other.__class__ == self.__class__ and \
+               (self.get_pk() != other.get_pk() or self.get_pk() is None)
+
     def get_field_dict(self):
         field_dict = {}
 
@@ -2680,3 +2752,38 @@ class Model(object):
 
         for field_name in fields:
             setattr(self, field_name, getattr(obj, field_name))
+
+def find_subclasses(klass, include_self=False):
+    accum = []
+    for child in klass.__subclasses__():
+        accum.extend(find_subclasses(child, True))
+    if include_self:
+        accum.append(klass)
+    return accum
+
+def create_model_tables(models, **create_table_kwargs):
+    """Create tables for all given models (in the right order)."""
+    for m in sort_models_topologically(models):
+        m.create_table(**create_table_kwargs)
+
+def drop_model_tables(models, **drop_table_kwargs):
+    """Drop tables for all given models (in the right order)."""
+    for m in reversed(sort_models_topologically(models)):
+        m.drop_table(**drop_table_kwargs)
+
+def sort_models_topologically(models):
+    """Sort models topologically so that parents will precede children."""
+    models = set(models)
+    seen = set()
+    ordering = []
+    def dfs(model):
+        if model in models and model not in seen:
+            seen.add(model)
+            for child_model in model._meta.reverse_relations.values():
+                dfs(child_model)
+            ordering.append(model)  # parent will follow descendants
+    # order models by name and table initially to guarantee a total ordering
+    names = lambda m: (m._meta.model_name, m._meta.db_table)
+    for m in sorted(models, key=names, reverse=True):
+        dfs(m)
+    return list(reversed(ordering))  # want parents first in output ordering
