@@ -100,6 +100,7 @@ class PgDB(DB):
         25: 'TextField',
         700: 'FloatField',
         701: 'FloatField',
+        1042: 'CharField', # blank-padded CHAR
         1043: 'CharField',
         1082: 'DateField',
         1114: 'DateTimeField',
@@ -107,6 +108,7 @@ class PgDB(DB):
         1083: 'TimeField',
         1266: 'TimeField',
         1700: 'DecimalField',
+        2950: 'TextField', # UUID
     }
 
     def get_conn_class(self):
@@ -255,8 +257,12 @@ def get_db(engine):
 
 def introspect(engine, database, **connect):
     db = get_db(engine)
+    schema = connect.pop('schema', None)
     db.connect(database, **connect)
 
+    if schema:
+        db.conn.set_search_path(*schema.split(','))
+
     tables = db.get_tables()
 
     models = {}
@@ -334,9 +340,10 @@ if __name__ == '__main__':
     ao('-u', '--user', dest='user')
     ao('-P', '--password', dest='password')
     ao('-e', '--engine', dest='engine', default='postgresql')
+    ao('-s', '--schema', dest='schema')
 
     options, args = parser.parse_args()
-    ops = ('host', 'port', 'user', 'password')
+    ops = ('host', 'port', 'user', 'password', 'schema')
     connect = dict((o, getattr(options, o)) for o in ops if getattr(options, o))
 
     if len(args) < 1:
